//
//  CATextLayer+NumberJump.m
//  BZNumberJumpDemo
//
//  Created by Bruce on 14-7-1.
//  Copyright (c) 2014年 com.Bruce.Number. All rights reserved.
//

#import "CACustomTextLayer.h"

#define kPointsNumber 100 // 即数字跳100次
#define kDurationTime 0.02 // 动画时间
#define kStartNumber  0   // 起始数字
#define kEndNumber    10000// 结束数字

@interface CACustomTextLayer()

@property (nonatomic, assign) NSInteger pointsNumber;
@property (nonatomic, assign) float durationTime;
@property (nonatomic, assign) NSInteger startNumber;
@property (nonatomic, assign) NSInteger endNumber;
@property (nonatomic, assign) NSInteger pointPlace;


@property (nonatomic, retain) NSMutableArray *numberPoints;//记录每次textLayer更改值的间隔时间及输出值。
@property (nonatomic, assign) NSInteger lastTime;
@property (nonatomic, assign) int indexNumber;

@property (nonatomic, assign) Point2D startPoint;
@property (nonatomic, assign) Point2D controlPoint1;
@property (nonatomic, assign) Point2D controlPoint2;
@property (nonatomic, assign) Point2D endPoint;

@end

@implementation CACustomTextLayer

- (instancetype)init
{
    self = [super init];
    if (!self) {
    }
    return self;
}

-(void)layoutSublayers
{
    [super layoutSublayers];
     self.contentsScale = 2;//解决模糊问题
}

- (void)cleanUpValue {
    _lastTime = 0;
    _indexNumber = 0;
    self.string = [NSString stringWithFormat:@"%.2ld",(long)_startNumber];
}

- (BOOL)isMatchWithRegexString:(NSString*)regexString endNumber:(NSString *)endNumber {
    return [[NSPredicate predicateWithFormat:@"SELF MATCHES %@", regexString] evaluateWithObject:endNumber];
}

- (void)jumpNumberWithDuration:(int)duration
                    fromNumber:(NSString*)startNumber
                      toNumber:(NSString*)endNumber
{
    if([self isMatchWithRegexString:@"^[+-]?\\d+(\\.\\d+)?$" endNumber:endNumber])
    {
        _durationTime = kDurationTime;
        _startNumber = [startNumber integerValue];
        _endNumber   = [endNumber integerValue];
        _pointPlace  = 0;
    }else{
        NSLog(@"数据异常");
        _durationTime = kDurationTime;
        _startNumber = 0;
        _endNumber   = 0;
        _pointPlace  = 0;
    }
    
    
    //TODO:1107新增判定小数点位置
    NSRange rang = [endNumber rangeOfString:@"."];
    if(rang.length > 0)
    {
        _pointPlace  = rang.location;
        NSMutableString *mtEndNumber = [[NSMutableString  alloc]initWithString:endNumber];
        [mtEndNumber deleteCharactersInRange:NSMakeRange(rang.location,1)];
        _endNumber   = [mtEndNumber integerValue];
    }else
    {
        _pointPlace = endNumber.length+1;
    }
    
    //TODO: 1103新增
    NSInteger changeValueBy = _endNumber - _startNumber;
    //数字 十以内的变化
    if (changeValueBy/100 < 1) {
        self.pointsNumber = changeValueBy;
    } else {
        self.pointsNumber = kPointsNumber;
    }
    
    [self cleanUpValue];
    [self initPoints];
    [self 循环递归数字跃动实现];
}


//保留两位小数
//千分位
-(NSString *)convert:(double)number
{
    NSNumberFormatter *numberFormatter = [[NSNumberFormatter alloc] init] ;
    [numberFormatter setPositiveFormat:@"###,##0;"];//###,##0.00
    NSString *formattedNumberString = [numberFormatter stringFromNumber:[NSNumber numberWithDouble:number]];
    return formattedNumberString;
}


- (void)jumpNumber {
    //    [self jumpNumberWithDuration:kDurationTime fromNumber:kStartNumber toNumber:kEndNumber];
}


- (void)initPoints {
    // 贝塞尔曲线 shijian
    [self initBezierPoints];
    Point2D bezierCurvePoints[4] = {_startPoint, _controlPoint1, _controlPoint2, _endPoint};
    _numberPoints = [[NSMutableArray alloc] init];
    float dt;
    dt = 1.0 / (self.pointsNumber - 1);
    for (int i = 0; i < self.pointsNumber; i++) {
        Point2D point = PointOnCubicBezier(bezierCurvePoints, i*dt);
        float durationTime = point.x * _durationTime;
        float value = point.y * (_endNumber - _startNumber) + _startNumber;
        [_numberPoints addObject:[NSArray arrayWithObjects:[NSNumber numberWithFloat:durationTime], [NSNumber numberWithFloat:value], nil]];
    }
}

- (void)initBezierPoints {
    // 可到http://cubic-bezier.com自定义贝塞尔曲线
    
    _startPoint.x = 0;
    _startPoint.y = 0;
    
    _controlPoint1.x = 0.65;
    _controlPoint1.y = 1.24;

    _controlPoint2.x = 1;
    _controlPoint2.y = 0.9;

//    _controlPoint1.x = 0.25;
//    _controlPoint1.y = 0.1;
//    
//    _controlPoint2.x = 0.25;
//    _controlPoint2.y = 1;
    
    _endPoint.x = 1;
    _endPoint.y = 1;
}

//循环递归调用本方法递增
- (void)循环递归数字跃动实现 {
    if (_indexNumber >= self.pointsNumber)
    {
        NSMutableString *temp = [NSMutableString stringWithFormat:@"%@",[NSString stringWithFormat:@"%.0ld",(long)_endNumber]];
        if (temp.length >= _pointPlace)
        {
            [temp insertString:@"." atIndex:_pointPlace];
        }
        self.string =[self convert:[temp doubleValue]];
        return;
    } else
    {
        NSArray *pointValues = [_numberPoints objectAtIndex:_indexNumber];
        _indexNumber++;
        float value = [(NSNumber *)[pointValues objectAtIndex:1] floatValue];// 有时要改成floatValue
        float currentTime = [(NSNumber *)[pointValues objectAtIndex:0] floatValue];
        float timeDuration = currentTime - _lastTime;
        _lastTime = currentTime;
        NSMutableString *temp = [NSMutableString stringWithFormat:@"%@",[NSString stringWithFormat:@"%.0f",value]];
        if (temp.length >= _pointPlace)
        {
            [temp insertString:@"." atIndex:_pointPlace];
        }
        self.string =[self convert:[temp doubleValue]];
        NSTimer *timer = [NSTimer scheduledTimerWithTimeInterval:timeDuration
                                                          target:self
                                                        selector:@selector(循环递归数字跃动实现) userInfo:nil
                                                         repeats:NO];
        [[NSRunLoop currentRunLoop] addTimer:timer forMode:NSRunLoopCommonModes];
    }
}


@end

