//
//  GradientView.m
//  DashboardDemo
//
//  Created by sunhong on 16/8/8.
//  Copyright © 2016年 sunhong. All rights reserved.
//

#import "GradientView.h"

static const CGFloat kTimerInterval = 0.03f;

static const NSInteger kPurePercent = 25;

@interface GradientView ()

@property (nonatomic, strong) CAGradientLayer * gradientLayer;

@property (nonatomic, strong) NSMutableArray * allColors;

@property (nonatomic, strong) NSArray * colors1;

@property (nonatomic, strong) NSArray * colors2;

@property (nonatomic, strong) NSArray * colors3;

@property (nonatomic, strong) NSArray * colors4;

@property (nonatomic, assign) NSInteger bgCount;

@property (nonatomic, assign) NSInteger changeCount;

@property (nonatomic, strong) NSTimer * bgTimer;

@property (nonatomic, assign) CGFloat totalAnimationTime;

@end

@implementation GradientView

- (NSMutableArray *)allColors {
    if (!_allColors) {
        _allColors = [NSMutableArray array];
    }
    return _allColors;
}

- (NSArray *)colors1 {
    if (_colors1.count == 0) {
        _colors1 = [NSArray arrayWithObjects:
                    (id)[[UIColor colorWithRed:1.000 green:0.679 blue:0.650 alpha:1.000] CGColor],
                    (id)[[UIColor colorWithRed:1.000 green:0.398 blue:0.362 alpha:1.000] CGColor],
                    (id)[[UIColor colorWithRed:1.000 green:0.276 blue:0.294 alpha:1.000] CGColor], nil];
    }
    return _colors1;
}

- (NSArray *)colors2 {
    if (_colors2.count == 0) {
        _colors2 = [NSArray arrayWithObjects:
                    (id)[[UIColor colorWithRed:1.000 green:0.843 blue:0.681 alpha:1.000] CGColor],
                    (id)[[UIColor colorWithRed:1.000 green:0.686 blue:0.383 alpha:1.000] CGColor],
                    (id)[[UIColor colorWithRed:1.000 green:0.620 blue:0.066 alpha:1.000] CGColor], nil];
    }
    return _colors2;
}

- (NSArray *)colors3 {
    if (_colors3.count == 0) {
        _colors3 = [NSArray arrayWithObjects:
                    (id)[[UIColor colorWithRed:0.672 green:1.000 blue:0.652 alpha:1.000] CGColor],
                    (id)[[UIColor colorWithRed:0.450 green:1.000 blue:0.406 alpha:1.000] CGColor],
                    (id)[[UIColor colorWithRed:0.139 green:1.000 blue:0.111 alpha:1.000] CGColor], nil];
    }
    return _colors3;
}

- (NSArray *)colors4 {
    if (_colors4.count == 0) {
        _colors4 = [NSArray arrayWithObjects:
                    (id)[[UIColor colorWithRed:0.664 green:0.693 blue:1.000 alpha:1.000] CGColor],
                    (id)[[UIColor colorWithRed:0.517 green:0.541 blue:1.000 alpha:1.000] CGColor],
                    (id)[[UIColor colorWithRed:0.259 green:0.304 blue:1.000 alpha:1.000] CGColor], nil];
    }
    return _colors4;
}

- (CAGradientLayer *)gradientLayer {
    
    if (_gradientLayer == nil) {
        CAGradientLayer *gradientLayer =  [CAGradientLayer layer];
        gradientLayer.frame = self.bounds;
        gradientLayer.colors = self.colors1;
        [gradientLayer setLocations:@[@0.3, @0.7, @1 ]];
        [gradientLayer setStartPoint:CGPointMake(0.5, 1)];
        [gradientLayer setEndPoint:CGPointMake(0.5, 0)];
        _gradientLayer = gradientLayer;
    }
    return _gradientLayer;
}

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        self.backgroundColor = [UIColor clearColor];
        
        self.allColors = [NSMutableArray arrayWithArray:@[self.colors1,self.colors2,self.colors3,self.colors4]];
        
        [self.layer addSublayer:self.gradientLayer];
        
    }
    return self;
}

#pragma mark - set

- (void)setPercent:(CGFloat)percent {
    _percent = percent;
    self.bgCount = percent / kPurePercent + 1;//个数不能超出colors的个数
    self.totalAnimationTime = kTimerInterval * percent;
    NSTimeInterval time = self.totalAnimationTime / self.bgCount;
    
    self.bgTimer = [NSTimer timerWithTimeInterval:time
                                              target:self
                                            selector:@selector(bgAnimation)
                                            userInfo:nil
                                             repeats:YES];
    [[NSRunLoop currentRunLoop] addTimer:_bgTimer forMode:NSRunLoopCommonModes];
}

- (void)bgAnimation {
    
    if (self.bgCount > self.allColors.count) {
        NSLog(@"颜色数组错误");
        [self.bgTimer invalidate];
        return;
    }
    
    _changeCount++;
    if (_changeCount == _bgCount) {
        _changeCount = 0;
        [self.bgTimer invalidate];
        return;
    }
    
    self.gradientLayer.colors = self.allColors[(self.changeCount)];
    [self setNeedsLayout];
}


@end
